package sun.java2d.pipe;

import java.awt.MultipleGradientPaint;
import java.awt.geom.Rectangle2D;
import sun.java2d.SurfaceData;
import sun.java2d.loops.CompositeType;
import java.awt.geom.NoninvertibleTransformException;
import sun.awt.image.PixelConverter;
import java.awt.geom.Point2D;
import java.awt.Color;
import java.awt.geom.AffineTransform;
import java.awt.TexturePaint;
import java.awt.RadialGradientPaint;
import java.awt.LinearGradientPaint;
import java.awt.GradientPaint;
import java.awt.Paint;
import sun.java2d.SunGraphics2D;

public class BufferedPaints
{
    public static final int MULTI_MAX_FRACTIONS = 12;
    
    public BufferedPaints() {
    }
    
    static void setPaint(RenderQueue renderQueue, SunGraphics2D sunGraphics2D, Paint paint, int n) {
        if (sunGraphics2D.paintState <= 1) {
            setColor(renderQueue, sunGraphics2D.pixel);
        }
        else {
            int n2 = (n & 0x2) != 0x0;
            switch (sunGraphics2D.paintState) {
                case 2: {
                    setGradientPaint(renderQueue, sunGraphics2D, (GradientPaint)paint, (boolean)n2);
                    break;
                }
                case 3: {
                    setLinearGradientPaint(renderQueue, sunGraphics2D, (LinearGradientPaint)paint, (boolean)n2);
                    break;
                }
                case 4: {
                    setRadialGradientPaint(renderQueue, sunGraphics2D, (RadialGradientPaint)paint, (boolean)n2);
                    break;
                }
                case 5: {
                    setTexturePaint(renderQueue, sunGraphics2D, (TexturePaint)paint, (boolean)n2);
                    break;
                }
            }
        }
    }
    
    static void resetPaint(RenderQueue renderQueue) {
        renderQueue.ensureCapacity(4);
        renderQueue.getBuffer().putInt(100);
    }
    
    private static void setColor(RenderQueue renderQueue, int n) {
        renderQueue.ensureCapacity(8);
        RenderBuffer buffer = renderQueue.getBuffer();
        buffer.putInt(101);
        buffer.putInt(n);
    }
    
    private static void setGradientPaint(RenderQueue renderQueue, AffineTransform affineTransform, Color color, Color color2, Point2D point2D, Point2D point2D2, boolean b, boolean b2) {
        PixelConverter instance = PixelConverter.ArgbPre.instance;
        int rgbToPixel = instance.rgbToPixel(color.getRGB(), null);
        int rgbToPixel2 = instance.rgbToPixel(color2.getRGB(), null);
        double x = point2D.getX();
        double y = point2D.getY();
        affineTransform.translate(x, y);
        double vecx = point2D2.getX() - x;
        double vecy = point2D2.getY() - y;
        double sqrt = Math.sqrt(vecx * vecx + vecy * vecy);
        affineTransform.rotate(vecx, vecy);
        affineTransform.scale(2.0 * sqrt, 1.0);
        affineTransform.translate(-0.25, 0.0);
        double scaleX;
        double shearX;
        double translateX;
        try {
            affineTransform.invert();
            scaleX = affineTransform.getScaleX();
            shearX = affineTransform.getShearX();
            translateX = affineTransform.getTranslateX();
        }
        catch (NoninvertibleTransformException ex) {
            shearX = (scaleX = (translateX = 0.0));
        }
        renderQueue.ensureCapacityAndAlignment(44, 12);
        RenderBuffer buffer = renderQueue.getBuffer();
        buffer.putInt(102);
        buffer.putInt(b2 ? 1 : 0);
        buffer.putInt(b ? 1 : 0);
        buffer.putDouble(scaleX).putDouble(shearX).putDouble(translateX);
        buffer.putInt(rgbToPixel).putInt(rgbToPixel2);
    }
    
    private static void setGradientPaint(RenderQueue renderQueue, SunGraphics2D sunGraphics2D, GradientPaint gradientPaint, boolean b) {
        setGradientPaint(renderQueue, (AffineTransform)sunGraphics2D.transform.clone(), gradientPaint.getColor1(), gradientPaint.getColor2(), gradientPaint.getPoint1(), gradientPaint.getPoint2(), gradientPaint.isCyclic(), b);
    }
    
    private static void setTexturePaint(RenderQueue renderQueue, SunGraphics2D sunGraphics2D, TexturePaint texturePaint, boolean b) {
        SurfaceData sourceSurfaceData = sunGraphics2D.surfaceData.getSourceSurfaceData(texturePaint.getImage(), 0, CompositeType.SrcOver, null);
        int n = sunGraphics2D.interpolationType != 1;
        AffineTransform affineTransform = (AffineTransform)sunGraphics2D.transform.clone();
        Rectangle2D anchorRect = texturePaint.getAnchorRect();
        affineTransform.translate(anchorRect.getX(), anchorRect.getY());
        affineTransform.scale(anchorRect.getWidth(), anchorRect.getHeight());
        double scaleX;
        double shearX;
        double translateX;
        double shearY;
        double scaleY;
        double translateY;
        try {
            affineTransform.invert();
            scaleX = affineTransform.getScaleX();
            shearX = affineTransform.getShearX();
            translateX = affineTransform.getTranslateX();
            shearY = affineTransform.getShearY();
            scaleY = affineTransform.getScaleY();
            translateY = affineTransform.getTranslateY();
        }
        catch (NoninvertibleTransformException ex) {
            shearX = (scaleX = (translateX = (shearY = (scaleY = (translateY = 0.0)))));
        }
        renderQueue.ensureCapacityAndAlignment(68, 12);
        RenderBuffer buffer = renderQueue.getBuffer();
        buffer.putInt(105);
        buffer.putInt(b ? 1 : 0);
        buffer.putInt(n ? 1 : 0);
        buffer.putLong(sourceSurfaceData.getNativeOps());
        buffer.putDouble(scaleX).putDouble(shearX).putDouble(translateX);
        buffer.putDouble(shearY).putDouble(scaleY).putDouble(translateY);
    }
    
    public static int convertSRGBtoLinearRGB(int n) {
        float n2 = (float)n / 255.0f;
        float n3;
        if (n2 <= 0.04045f) {
            n3 = n2 / 12.92f;
        }
        else {
            n3 = (float)Math.pow(((double)n2 + 0.055) / 1.055, 2.4);
        }
        return Math.round(n3 * 255.0f);
    }
    
    private static int colorToIntArgbPrePixel(Color color, boolean b) {
        int rgb = color.getRGB();
        if (!b && rgb >> 24 == -1) {
            return rgb;
        }
        int n = rgb >>> 24;
        int convertSRGBtoLinearRGB = rgb >> 16 & 0xFF;
        int convertSRGBtoLinearRGB2 = rgb >> 8 & 0xFF;
        int convertSRGBtoLinearRGB3 = rgb & 0xFF;
        if (b) {
            convertSRGBtoLinearRGB = convertSRGBtoLinearRGB(convertSRGBtoLinearRGB);
            convertSRGBtoLinearRGB2 = convertSRGBtoLinearRGB(convertSRGBtoLinearRGB2);
            convertSRGBtoLinearRGB3 = convertSRGBtoLinearRGB(convertSRGBtoLinearRGB3);
        }
        int n2 = n + (n >> 7);
        return n << 24 | convertSRGBtoLinearRGB * n2 >> 8 << 16 | convertSRGBtoLinearRGB2 * n2 >> 8 << 8 | convertSRGBtoLinearRGB3 * n2 >> 8;
    }
    
    private static int[] convertToIntArgbPrePixels(Color[] array, boolean b) {
        int[] array2 = new int[array.length];
        for (int i = 0; i < array.length; ++i) {
            array2[i] = colorToIntArgbPrePixel(array[i], b);
        }
        return array2;
    }
    
    private static void setLinearGradientPaint(RenderQueue renderQueue, SunGraphics2D sunGraphics2D, LinearGradientPaint linearGradientPaint, boolean b) {
        int n = linearGradientPaint.getColorSpace() == MultipleGradientPaint.ColorSpaceType.LINEAR_RGB;
        Color[] colors = linearGradientPaint.getColors();
        int length = colors.length;
        Point2D startPoint = linearGradientPaint.getStartPoint();
        Point2D endPoint = linearGradientPaint.getEndPoint();
        AffineTransform transform = linearGradientPaint.getTransform();
        transform.preConcatenate(sunGraphics2D.transform);
        if (n == 0 && length == 2 && linearGradientPaint.getCycleMethod() != MultipleGradientPaint.CycleMethod.REPEAT) {
            setGradientPaint(renderQueue, transform, colors[0], colors[1], startPoint, endPoint, linearGradientPaint.getCycleMethod() != MultipleGradientPaint.CycleMethod.NO_CYCLE, b);
            return;
        }
        int ordinal = linearGradientPaint.getCycleMethod().ordinal();
        float[] fractions = linearGradientPaint.getFractions();
        int[] convertToIntArgbPrePixels = convertToIntArgbPrePixels(colors, (boolean)n);
        double x = startPoint.getX();
        double y = startPoint.getY();
        transform.translate(x, y);
        double vecx = endPoint.getX() - x;
        double vecy = endPoint.getY() - y;
        double sqrt = Math.sqrt(vecx * vecx + vecy * vecy);
        transform.rotate(vecx, vecy);
        transform.scale(sqrt, 1.0);
        float n2;
        float n3;
        float n4;
        try {
            transform.invert();
            n2 = (float)transform.getScaleX();
            n3 = (float)transform.getShearX();
            n4 = (float)transform.getTranslateX();
        }
        catch (NoninvertibleTransformException ex) {
            n3 = (n2 = (n4 = 0.0f));
        }
        renderQueue.ensureCapacity(32 + length * 4 * 2);
        RenderBuffer buffer = renderQueue.getBuffer();
        buffer.putInt(103);
        buffer.putInt(b ? 1 : 0);
        buffer.putInt(n ? 1 : 0);
        buffer.putInt(ordinal);
        buffer.putInt(length);
        buffer.putFloat(n2);
        buffer.putFloat(n3);
        buffer.putFloat(n4);
        buffer.put(fractions);
        buffer.put(convertToIntArgbPrePixels);
        return;
    }
    
    private static void setRadialGradientPaint(RenderQueue renderQueue, SunGraphics2D sunGraphics2D, RadialGradientPaint radialGradientPaint, boolean b) {
        int n = radialGradientPaint.getColorSpace() == MultipleGradientPaint.ColorSpaceType.LINEAR_RGB;
        int ordinal = radialGradientPaint.getCycleMethod().ordinal();
        float[] fractions = radialGradientPaint.getFractions();
        Color[] colors = radialGradientPaint.getColors();
        int length = colors.length;
        int[] convertToIntArgbPrePixels = convertToIntArgbPrePixels(colors, (boolean)n);
        Point2D centerPoint = radialGradientPaint.getCenterPoint();
        Point2D focusPoint = radialGradientPaint.getFocusPoint();
        float radius = radialGradientPaint.getRadius();
        double x = centerPoint.getX();
        double y = centerPoint.getY();
        double x2 = focusPoint.getX();
        double y2 = focusPoint.getY();
        AffineTransform transform = radialGradientPaint.getTransform();
        transform.preConcatenate(sunGraphics2D.transform);
        Point2D transform2 = transform.transform(focusPoint, focusPoint);
        transform.translate(x, y);
        transform.rotate(x2 - x, y2 - y);
        transform.scale((double)radius, (double)radius);
        try {
            transform.invert();
        }
        catch (Exception ex) {
            transform.setToScale(0.0, 0.0);
        }
        double min = Math.min(transform.transform(transform2, transform2).getX(), 0.99);
        renderQueue.ensureCapacity(48 + length * 4 * 2);
        RenderBuffer buffer = renderQueue.getBuffer();
        buffer.putInt(104);
        buffer.putInt(b ? 1 : 0);
        buffer.putInt(n ? 1 : 0);
        buffer.putInt(length);
        buffer.putInt(ordinal);
        buffer.putFloat((float)transform.getScaleX());
        buffer.putFloat((float)transform.getShearX());
        buffer.putFloat((float)transform.getTranslateX());
        buffer.putFloat((float)transform.getShearY());
        buffer.putFloat((float)transform.getScaleY());
        buffer.putFloat((float)transform.getTranslateY());
        buffer.putFloat((float)min);
        buffer.put(fractions);
        buffer.put(convertToIntArgbPrePixels);
    }
}
